@@ -33,41 +33,60 @@ class MeshArtist(PlotterArtist, MeshArtist):
     ----------
     mesh : :class:`compas.datastructures.Mesh`
         A COMPAS mesh.
-    vertices : list of int, optional
-        A list of vertex identifiers.
-        Default is ``None``, in which case all vertices are drawn.
-    edges : list, optional
-        A list of edge keys (as uv pairs) identifying which edges to draw.
-        The default is ``None``, in which case all edges are drawn.
-    faces : list, optional
-        A list of face identifiers.
-        The default is ``None``, in which case all faces are drawn.
-    vertexcolor : rgb-tuple or dict of rgb-tuples, optional
-        The color specification for the vertices.
-    edgecolor : rgb-tuple or dict of rgb-tuples, optional
-        The color specification for the edges.
-    facecolor : rgb-tuple or dict of rgb-tuples, optional
-        The color specification for the faces.
+    vertices : list[int], optional
+        Selection of vertex identifiers.
+        Default is None, in which case all vertices are drawn.
+    edges : list[tuple[int, int]], optional
+        Selection of edge identifiers.
+        The default is None, in which case all edges are drawn.
+    faces : list[int], optional
+        Selection of face identifiers.
+        The default is None, in which case all faces are drawn.
+    vertexcolor : tuple[float, float, float] or dict[int, tuple[float, float, float]], optional
+        Color specification for the vertices.
+    edgecolor : tuple[float, float, float] or dict[tuple[int, int], tuple[float, float, float]], optional
+        Color specification for the edges.
+    facecolor : tuple[float, float, float] or dict[int, tuple[float, float, float]], optional
+        Color specification for the faces.
     show_vertices : bool, optional
+        If True, draw the vertices of the mesh.
     show_edges : bool, optional
+        If True, draw the edges of the mesh.
     show_faces : bool, optional
+        If True, draw the faces of the mesh.
     vertexsize : int, optional
+        Size of the vertices.
+    vertextext : str or dict[int, str], optional
+        Labels for the vertices.
+    edgetext : str or dict[tuple[int, int], str], optional
+        Labels for the edges.
+    facetext : str or dict[int, str], optional
+        Labels for the faces.
     sizepolicy : {'relative', 'absolute'}, optional
+        The policy for sizing the vertices.
+        If ``'relative'``, the value of `vertexsize` is scaled by the number of vertices.
+        If ``'absolute'``, the value of `vertexsize` is scaled by the resolution of the plotter (:attr:MeshArtist.plotter.dpi).
+    zorder : int, optional
+        The base stacking order of the components of the mesh on the canvas.
+    **kwargs : dict, optional
+        Additional keyword arguments.
+        See :class:`compas_plotters.artists.PlotterArtist` and :class:`compas.artists.MeshArtist` for more info.
 
     Attributes
     ----------
-    vertexcollection : :class:`PatchCollection`
-        The collection containing the vertices.
-    edgecollection : :class:`LineCollection`
-        The collection containing the edges.
-    facecollection : :class:`PatchCollection`
-        The collection containing the faces.
+    halfedges : list[tuple[int, int]]
+        The halfedges to include in the drawing.
+    vertex_size : dict[int, float]
+        Mapping between vertex identifiers and vertex sizes.
+    halfedge_color : dict[tuple[int, int], tuple[float, float, float]]
+        Mapping between halfedge identifiers and halfedge colors.
+    zorder_faces : int, read-only
+        The stacking order of the faces relative to the base stacking order of the mesh.
+    zorder_edges : int, read-only
+        The stacking order of the edges relative to the base stacking order of the mesh.
+    zorder_vertices : int, read-only
+        The stacking order of the vertices relative to the base stacking order of the mesh.
 
-    Class Attributes
-    ----------------
-    zorder_vertices : int
-    zorder_edges : int
-    zorder_faces : int
     """
 
     default_halfedgecolor = (0.7, 0.7, 0.7)
@@ -182,22 +201,50 @@ class MeshArtist(PlotterArtist, MeshArtist):
     # ==============================================================================
 
     def clear_vertices(self) -> None:
+        """Clear the current vertices from the canvas.
+
+        Returns
+        -------
+        None
+
+        """
         if self._vertexcollection:
             self._vertexcollection.remove()
         self._vertexcollection = None
 
     def clear_edges(self) -> None:
+        """Clear the current edges from the canvas.
+
+        Returns
+        -------
+        None
+
+        """
         if self._edgecollection:
             self._edgecollection.remove()
         self._edgecollection = None
 
     def clear_halfedges(self) -> None:
+        """Clear the current halfedges from the canvas.
+
+        Returns
+        -------
+        None
+
+        """
         if self._halfedgecollection:
             for artist in self._halfedgecollection:
                 artist.remove()
         self._halfedgecollection = None
 
     def clear_faces(self) -> None:
+        """Clear the current faces from the canvas.
+
+        Returns
+        -------
+        None
+
+        """
         if self._facecollection:
             self._facecollection.remove()
         self._facecollection = None
@@ -214,21 +261,26 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        vertices : list of int, optional
+        vertices : list[int], optional
             A list of vertex identifiers.
-            Default is ``None``, in which case all vertices are drawn.
-        edges : list, optional
+            Default is None, in which case all vertices are drawn.
+        edges : list[tuple[int, int]], optional
             A list of edge keys (as uv pairs) identifying which edges to draw.
-            The default is ``None``, in which case all edges are drawn.
-        faces : list, optional
+            The default is None, in which case all edges are drawn.
+        faces : list[int], optional
             A list of face identifiers.
-            The default is ``None``, in which case all faces are drawn.
-        vertexcolor : rgb-tuple or dict of rgb-tuples, optional
+            The default is None, in which case all faces are drawn.
+        vertexcolor : rgb-tuple or dict[int, rgb-tuple], optional
             The color specification for the vertices.
-        edgecolor : rgb-tuple or dict of rgb-tuples, optional
+        edgecolor : rgb-tuple or dict[tuple[int, int], rgb-tuple], optional
             The color specification for the edges.
-        facecolor : rgb-tuple or dict of rgb-tuples, optional
+        facecolor : rgb-tuple or dict[int, rgb-tuple], optional
             The color specification for the faces.
+
+        Returns
+        -------
+        None
+
         """
         self.clear()
         if self.show_vertices:
@@ -238,6 +290,9 @@ class MeshArtist(PlotterArtist, MeshArtist):
         if self.show_faces:
             self.draw_faces(faces=faces, color=facecolor)
 
+    def draw_mesh(self):
+        raise NotImplementedError
+
     def draw_vertices(self,
                       vertices: Optional[List[int]] = None,
                       color: Optional[Union[str, Color, List[Color], Dict[int, Color]]] = None
@@ -246,15 +301,16 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        vertices : list of int, optional
+        vertices : list[int], optional
             A list of vertex identifiers.
-            Default is ``None``, in which case all vertices are drawn.
-        color : rgb-tuple or dict of rgb-tuples, optional
+            Default is None, in which case all vertices are drawn.
+        color : rgb-tuple or dict[int, rgb-tuple], optional
             The color specification for the vertices.
 
         Returns
         -------
         None
+
         """
         self.clear_vertices()
 
@@ -293,15 +349,16 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        edges : list, optional
+        edges : list[tuple[int, int]], optional
             A list of edge keys (as uv pairs) identifying which edges to draw.
-            The default is ``None``, in which case all edges are drawn.
-        color : rgb-tuple or dict of rgb-tuples, optional
+            The default is None, in which case all edges are drawn.
+        color : rgb-tuple or dict[tuple[int, int], rgb-tuple], optional
             The color specification for the edges.
 
         Returns
         -------
         None
+
         """
         self.clear_edges()
         if edges:
@@ -313,9 +370,10 @@ class MeshArtist(PlotterArtist, MeshArtist):
         colors = []
         widths = []
         for edge in self.edges:
+            u, v = edge
             lines.append([self.vertex_xyz[edge[0]][:2], self.vertex_xyz[edge[1]][:2]])
-            colors.append(self.edge_color.get(edge, self.default_edgecolor))
-            widths.append(self.edge_width.get(edge, self.default_edgewidth))
+            colors.append(self.edge_color.get(edge, self.edge_color.get((v, u), self.default_edgecolor)))
+            widths.append(self.edge_width.get(edge, self.edge_width.get((v, u), self.default_edgewidth)))
 
         collection = LineCollection(
             lines,
@@ -339,15 +397,16 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        edges : list, optional
+        edges : list[tuple[int, int]], optional
             A list of halfedges to draw.
-            The default is ``None``, in which case all halfedges are drawn.
-        color : rgb-tuple or dict of rgb-tuples, optional
+            The default is None, in which case all halfedges are drawn.
+        color : rgb-tuple or dict[tuple[int, int], rgb-tuple], optional
             The color specification for the halfedges.
 
         Returns
         -------
         None
+
         """
         self.clear_halfedges()
         self._halfedgecollection = []
@@ -393,15 +452,16 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        faces : list, optional
+        faces : list[int], optional
             A list of face identifiers.
-            The default is ``None``, in which case all faces are drawn.
-        color : rgb-tuple or dict of rgb-tuples, optional
+            The default is None, in which case all faces are drawn.
+        color : rgb-tuple or dict[int, rgb-tuple], optional
             The color specification for the faces.
 
         Returns
         -------
         None
+
         """
         self.clear_faces()
         if faces:
@@ -437,13 +497,13 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        text : dict of int to str, optional
+        text : dict[int, str], optional
             A vertex-label map.
-            If not text dict is provided, the vertex identifiers are drawn.
 
         Returns
         -------
         None
+
         """
         if self._vertexlabelcollection:
             for artist in self._vertexlabelcollection:
@@ -465,7 +525,7 @@ class MeshArtist(PlotterArtist, MeshArtist):
             artist = self.plotter.axes.text(
                 x, y,
                 f'{text}',
-                fontsize=12,
+                fontsize=self.plotter.fontsize,
                 family='monospace',
                 ha='center', va='center',
                 zorder=10000,
@@ -480,12 +540,13 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        text : dict of tuple of int to str
+        text : dict[tuple[int, int], str]
             An edge-label map.
 
         Returns
         -------
         None
+
         """
         if self._edgelabelcollection:
             for artist in self._edgelabelcollection:
@@ -496,7 +557,8 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         labels = []
         for edge in self.edges:
-            text = self.edge_text.get(edge, None)
+            u, v = edge
+            text = self.edge_text.get(edge, self.edge_text.get((v, u), None))
             if text is None:
                 continue
 
@@ -507,11 +569,12 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
             artist = self.plotter.axes.text(
                 x, y, f'{text}',
-                fontsize=12,
+                fontsize=self.plotter.fontsize,
                 family='monospace',
                 ha='center', va='center',
                 zorder=10000,
-                color=(0, 0, 0)
+                color=(0, 0, 0),
+                bbox=dict(boxstyle='round, pad=0.3', facecolor=(1, 1, 1), edgecolor=None, linewidth=0)
             )
             labels.append(artist)
 
@@ -522,12 +585,13 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
         Parameters
         ----------
-        text : dict of int to str
+        text : dict[int, str]
             A face-label map.
 
         Returns
         -------
         None
+
         """
         if self._facelabelcollection:
             for artist in self._facelabelcollection:
@@ -546,21 +610,41 @@ class MeshArtist(PlotterArtist, MeshArtist):
 
             artist = self.plotter.axes.text(
                 x, y, f'{text}',
-                fontsize=12,
+                fontsize=self.plotter.fontsize,
                 family='monospace',
                 ha='center', va='center',
                 zorder=10000,
                 color=(0, 0, 0),
-                bbox=dict(boxstyle='circle, pad=0.7', facecolor=(1, 1, 1), edgecolor=(0.5, 0.5, 0.5), linestyle=':')
+                bbox=dict(boxstyle='circle, pad=0.5', facecolor=(1, 1, 1), edgecolor=(0.5, 0.5, 0.5), linestyle=':')
             )
             labels.append(artist)
 
         self._facelabelcollection = labels
 
     def redraw(self) -> None:
+        """Redraw the mesh using the current geometry.
+
+        Returns
+        -------
+        None
+
+        """
         pass
 
     def update_vertexcolors(self, colors):
+        """Update the colors of the vertices.
+
+        Parameters
+        ----------
+        colors : dict[int, tuple[float, float, float]]
+            Mapping between vertex identifiers and colors.
+            Missing vertices get the default color: :attr:`MeshArtist.default_vertexcolor`.
+
+        Returns
+        -------
+        None
+
+        """
         facecolors = []
         for vertex in self.vertices:
             if vertex in colors:
@@ -571,6 +655,19 @@ class MeshArtist(PlotterArtist, MeshArtist):
         self._vertexcollection.set_facecolors(facecolors)
 
     def update_edgecolors(self, colors):
+        """Update the colors of the edges.
+
+        Parameters
+        ----------
+        colors : dict[tuple[int, int], tuple[float, float, float]]
+            Mapping between edge identifiers and colors.
+            Missing edge get the default color: :attr:`MeshArtist.default_edgecolor`.
+
+        Returns
+        -------
+        None
+
+        """
         edgecolors = []
         for edge in self.edges:
             if edge in colors:
@@ -581,6 +678,19 @@ class MeshArtist(PlotterArtist, MeshArtist):
         self._edgecollection.set_colors(edgecolors)
 
     def update_edgewidths(self, widths):
+        """Update the widths of the edges.
+
+        Parameters
+        ----------
+        widths : dict[tuple[int, int], float]
+            Mapping between edge identifiers and linewidths.
+            Missing edges get the default edge linewidth: :attr:`MeshArtist.default_edgewidth`.
+
+        Returns
+        -------
+        None
+
+        """
         edgewidths = []
         for edge in self.edges:
             if edge in widths:
